unit QRDiagram;(*
    Szerzo : Bakos Sandor  e-mail: dr_saca@freemail.hu
    Datum : 2001.07.03

    OszlopDiagramm
    property Font                  A megjeleno szoveg tul.
    property RWidth : integer      A diagram magassaga
    property RHeight : integer     A digram szelessege
    property ViewText : Boolean    magyarazat ?
    property PerCent : Boolean     szazalek ?
    property RectSize : integer    magyarazat kocka merete
    property RectDist : integer    magyarazat kockak tavolsaga
    property DiagRectDist : integer magyarazat kocka tavolsaga a  diagramtol


*)

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QExtCtrls;

type
  TBSRDiagram = class(TGraphicControl)
  private
    { Private declarations }
    fRWidth  : integer;
    fRHeight : integer;
    fData : TList;
    fViewText : Boolean;
    fPerCent  : Boolean;
    fRectSize : integer;
    fRectDist : integer;
    fDiagRectDist : integer;
    procedure SetRWidth(Value : integer);
    procedure SetRHeight(Value : integer);
  protected
    { Protected declarations }
    procedure SetWidth;
  public
    { Public declarations }
    procedure Paint; override;
    constructor Create (AOwner:TComponent); override;
    destructor Destroy ; override;
    procedure AddElement( Size : longint; Text : string ; Color : TColor; Style : TBrushStyle);
    procedure UpdateElement(Element : integer; Size : longint; Text : string ; Color : TColor; Style : TBrushStyle);
    procedure DeleteElement( Value : Integer );
    procedure ClearElements;
  published
    { Published declarations }
    property Width default 200;
    property Height default 200;
    property Font;
    property RWidth : integer
             read fRWidth write SetRWidth default 200 ;
    property RHeight : integer
             read fRHeight write SetRHeight default 200 ;
    property ViewText : Boolean
             read fViewText write fViewText default false;
    property PerCent : Boolean
             read fPerCent write fPerCent default false;
    property RectSize : integer
             read fRectSize write fRectSize default 10;
    property RectDist : integer
             read fRectDist write fRectDist default 10;
    property DiagRectDist : integer
             read fDiagRectDist write fDiagRectDist default 10;

  end;


{type TCDiagramCategory = class (TObject);
     class function Name : string ;
     class function Description : string ;
   end;}



procedure Register;

implementation


type
    PDataRec = ^DataRec;
    DataRec = record            // Tarolt adatok :
      size  : longint;          //  megadott ertek
      width : integer;          // az osszeshez viszonyitott kor%
      text  : string;           //  jelmagyarazat szoveg
      color : TColor ;          // szin
      style : TBrushStyle;      // kitoltesi stilus
    end;



constructor TBSRDiagram.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  fData:=TList.Create;
  fRWidth:=100;
  fRHeight:=200;
  fViewText:=false;
  fPerCent:=false;
  fRectSize:=10;
  fRectDist:=10;
  fDiagRectDist:=10;
  Width:=fRWidth;
  Height:=fRHeight;
  ParentColor:=True;
end;


destructor TBSRDiagram.Destroy;
var i:integer;
    MyDataRec : PDataRec;
begin
  for i:=0 to fData.Count-1 do
  begin
      MyDataRec:=fData.Items[i];
      Dispose(MyDataRec);
  end;
  inherited Destroy;
end;


procedure TBSRDiagram.SetRWidth(Value : integer);
begin

  If Value<>fRWidth Then
  begin
     fRWidth:=Value;
     If fRWidth>Width Then
        Width:=fRWidth;
     SetWidth;
     Paint;
  end;

end;


procedure TBSRDiagram.SetRHeight(Value : integer);
begin

  If Value<>fRHeight Then
  begin
     fRHeight:=Value;
     Height:=fRHeight;
     SetWidth;
     Paint;
  end;

end;



procedure TBSRDiagram.SetWidth;
var fullsize, i : integer ;
    MyDataRec   : PDataRec ;
begin

 if fData.Count>1 then
 begin

    fullsize := 0;
    for i:=0 to fData.Count-1 do
      begin
        MyDataRec:=fData.Items[i];
        fullsize := fullsize + MyDataRec.size;
      end;

    for i:=0 to fData.Count-1 do
      begin
        MyDataRec := fData.Items[i];
        MyDataRec.width := Round( (fRHeight/100) *
                      ( MyDataRec.size / (fullsize / 100) ) );
      end;

    fullsize := 0;
    for i:=0 to fData.Count-1 do
      begin
        MyDataRec:=fData.Items[i];
        fullsize := fullsize + MyDataRec.width;
      end;


    If fullsize<>Height Then
      begin
        Height:=fullsize;
        fRHeight:=fullsize;
      end;
 end;

end;



procedure TBSRDiagram.AddElement( Size : longint; Text: string ;
                                Color : TColor; Style : TBrushStyle);
var MyDataRec:PDataRec;
begin
  New(MyDataRec);
  MyDataRec.size  := Size  ;
  MyDataRec.text  := Text  ;
  MyDataRec.color := Color ;
  MyDataRec.style := Style ;
  fData.Add(MyDataRec);
  SetWidth;
end;


procedure TBSRDiagram.UpdateElement(Element : integer; Size : longint; Text : string ;
                        Color : TColor; Style : TBrushStyle);
var MyDataRec:PDataRec;
begin

   If (Element<=fData.Count) and (fData.Count>0) and (Element>0) Then
   begin
      MyDataRec:=fData.Items[Element-1];
      MyDataRec.size  := Size  ;
      MyDataRec.text  := Text  ;
      MyDataRec.color := Color ;
      MyDataRec.style := Style ;
      SetWidth;
   end;

end;


procedure TBSRDiagram.DeleteElement(Value : integer);
var MyDataRec : PDataRec;
begin
   If (Value<=fData.Count) and (fData.Count>0) and (Value>0) Then
   begin
      MyDataRec:=fData.Items[Value-1];
      Dispose(MyDataRec);
      fData.Delete(Value-1);
      fData.Pack;
      SetWidth;
   end;
end;


procedure TBSRDiagram.ClearElements;
var i:integer;
    MyDataRec : PDataRec;
begin

  for i:=0 to fData.Count-1 do
  begin
      MyDataRec:=fData.Items[i];
      Dispose(MyDataRec);
  end;
  fData.Clear;

end;



procedure TBSRDiagram.Paint;
var i , aktpos,aktrow : integer;
    MyDataRec  : PDataRec;
begin


  If fData.Count<2 Then
  begin

      Canvas.Brush.Color:=Color;
      Canvas.Pen.Color:=Color;
      Canvas.Brush.Style:=bsSolid;
      Canvas.Rectangle(fRWidth+fDiagRectDist,0,
                       fRWidth+fDiagRectDist+(Width-(fRWidth+fDiagRectDist)),Height);
      Canvas.Pen.Color:=clBlack;

      Canvas.Brush.Color:=clWhite;
      Canvas.Brush.Style:=bsSolid;
      Canvas.Rectangle(0,0,fRWidth,fRHeight);

  end Else begin

      aktpos:=0;
      aktrow:=5;

      If fViewText or fPerCent Then
      begin
        Canvas.Brush.Color:=Color;
        Canvas.Pen.Color:=Color;
        Canvas.Brush.Style:=bsSolid;
        Canvas.Rectangle(fRWidth+fDiagRectDist,0,
                         fRWidth+fDiagRectDist+(Width-(fRWidth+fDiagRectDist)),Height);
        Canvas.Pen.Color:=clBlack;
      end;

      for i:=0 to  fData.Count-1 do
      begin
        MyDataRec:=fData.Items[i];
        Canvas.Brush.Color:=MyDataRec.color;
        Canvas.Brush.Style:=MyDataRec.style;
        Canvas.Rectangle(0,0+aktpos,fRWidth,aktpos+MyDataRec.width);
        aktpos := aktpos + MyDataRec.width;

        If fViewText or fPerCent Then
        begin
           Canvas.Font:=Font;
           Canvas.Rectangle(fRWidth+fDiagRectDist,aktrow,
                            fRWidth+fDiagRectDist+fRectSize,aktrow+fRectSize);

           If fViewText and not fPerCent Then
              Canvas.TextOut(fRWidth+fDiagRectDist+fRectSize+10,
                             aktrow,MyDataRec.text);

           If  fPerCent and not fViewText Then
              Canvas.TextOut(fRWidth+fDiagRectDist+fRectSize+10,
                             aktrow,IntToStr(Round(MyDataRec.width/(fRHeight/100)))+' %');

           If  fPerCent and fViewText Then
               Canvas.TextOut(fRWidth+fDiagRectDist+fRectSize+10,
                              aktrow,
                              MyDataRec.text+' '+IntToStr(Round(MyDataRec.width/(fRHeight/100)))+' %');

           aktrow := aktrow + fRectDist + fRectSize;
        end;
      end;

  end;

end;




procedure Register;
begin
  RegisterComponents('BSDiagrams', [TBSRDiagram]);
end;

end.
